#!/usr/local/sbin/charm-env python3
"""Charm actions."""

import subprocess
import sys
import traceback
sys.path.append("lib")

from charmhelpers.core import (
    hookenv,
    host,
    unitdata,
)  # noqa: disable=C0413

from charms.layer.livepatchserver import resource
from reactive.livepatchserver import check_config

SERVICE = "livepatch-server.service"

KV = unitdata.kv()


def fail(message, details=None):
    """Fail the action with specified message and optional details."""
    if details:
        hookenv.action_set(details)
    hookenv.action_fail(message)
    sys.exit(1)


def schema_upgrade(args):
    """Run the schema upgrade."""
    pgurl = KV.get('database.master.dsn', None)
    if not pgurl:
        raise Exception("postgres not configured")
    hookenv.function_log("Running schema upgrade.")
    output = subprocess.check_output([
        './resources/schema-upgrade',
        '--db', pgurl,
        'upgrade',
        './resources/schema-upgrades',
    ])
    hookenv.function_set({"output": output.decode('utf-8')})
    hookenv.function_log("Restarting service.")
    host.service_restart(SERVICE)


def restart(args):
    """Restart livepatch service."""
    hookenv.function_log("Restarting livepatch service.")
    host.service_restart(SERVICE)
    message = "✓ Livepatch server restarted."
    hookenv.status_set("active", message.encode('utf-8'))


def get_resource_token(args):
    """Get resource token using ua client data on machine."""
    contracts_url = resource.get_contracts_url()
    if not contracts_url:
        fail("Failed to get contracts url. Is ua client installed?")
    
    (machine_id, machine_token) = resource.get_machine_token_info()
    if not machine_id or not machine_token:
        fail("Failed to get machine token.")

    token_resource = resource.get_resource_token(machine_id, machine_token,
                                        contracts_url=contracts_url)
    if not token_resource:
        fail("Failed to retrieve resource token.")
    try:
        KV.set('sync_token', token_resource['resourceToken'])
        KV.set('sync_upstream', token_resource['entitlement']['directives']['remoteServer'])
        KV.flush()
    except KeyError:
        fail("Failed to parse resource information.")
    hookenv.status_set("active", "⌛ Config updated. Restarting.".encode('utf-8'))
    hookenv.function_set({"output": "Resource token acquired."})
    check_config()


ACTIONS = {
    "schema-upgrade": schema_upgrade,
    "restart": restart,
    "get-resource-token": get_resource_token,
}


def main(args):
    action_name = hookenv.action_name()
    try:
        action = ACTIONS[action_name]
    except KeyError:
        return "Action '{}' undefined".format(action_name)

    try:
        action(args)
    except Exception as e:
        fail(str(e), details={"traceback": traceback.format_exc()})


if __name__ == "__main__":
    sys.exit(main(sys.argv))
