""" Livepatch server config. """

import yaml
import os

from charmhelpers.core.hookenv import (
    is_leader,
    log,
)

FIELDS = [
    'burst_limit',
    'concurrency_limit',
    'dbconn_max',
    'dbconn_max_lifetime',
    'log_level',
    'patch_cache_on',
    'patch_cache_size',
    'patch_cache_ttl',
    'port',
    'psql_url',
    'report_cleanup_interval',
    'report_retention',
    'sync_token',
    'sync_upstream',
    'url_template',
    'deployment_id',
]

# Fields that  can be left unset.
OPTIONAL_FIELDS = [
    'auth_basic_users',
    'auth_lp_teams',
    'auth_sso_public_key',
    'auth_sso_location',
]

# Fields that will not be added to the config.
SILENT_FIELDS = [
    'auth_basic_users',
    'auth_lp_teams',
    'auth_sso_location',
    'auth_sso_public_key',
    'filestore_path',
    'patchstore',
    'port',
    'report_cleanup_interval',
    'report_retention',
    's3_access_key_id',
    's3_bucket',
    's3_endpoint',
    's3_region',
    's3_secret_key',
    's3_secure',
    'swift_apikey',
    'swift_auth_url',
    'swift_container_name',
    'swift_domain_name',
    'swift_region_name',
    'swift_tenant_name',
    'swift_username',
    'sync_flavors',
    'sync_interval',
    'sync_token',
    'sync_upstream',
]

class MissingConfigKey(Exception):
    def __init__(self, message):
        super()
        self.message = message

    def __str__(self):
        return self.message


class InvalidConfigKey(Exception):
    def __init__(self, message):
        super()
        self.message = message

    def __str__(self):
        return self.message


def check_and_write_config(cfg, file_name, permissions=0o644):
    """Validate config and write to yaml file."""
    lp_cfg = {
        'admin-auth': map_admin_auth(cfg),
        'patch-storage': map_storage_config(cfg),
        'addr': ':{}'.format(cfg.get('port')),
        'reports': map_reports_config(cfg),
        'leader': is_leader(),
    }

    sync_config = map_sync_config(cfg)
    if sync_config:
        lp_cfg['sync'] = sync_config

    for key in FIELDS + OPTIONAL_FIELDS:
        value = cfg.get(key, None)
        if value is None or value == "" and key not in OPTIONAL_FIELDS:
            raise MissingConfigKey("missing config setting {}".format(key))
        if key not in SILENT_FIELDS:
            lp_cfg[key] = value
    with open(file_name, 'w') as yaml_file:
        yaml.dump(lp_cfg, yaml_file)
    os.chmod(file_name, permissions)


def map_reports_config(cfg):
    scfg = {
        'db': {
            'enabled': True,
            'retention-days': cfg.get('report_retention'),
            'cleanup-interval': cfg.get('report_cleanup_interval'),
            'cleanup-row-limit': cfg.get('report_cleanup_row_limit'),
        }
    }
    return scfg


def map_sync_config(cfg):
    """Create patch synchronization section in config file."""
    token = cfg.get('sync_token', None)
    if not token:
        raise MissingConfigKey("sync_token not set")
    upstream_url = cfg.get('sync_upstream')
    interval = cfg.get('sync_interval')
    flavors = cfg.get('sync_flavors')
    send_machine_reports = cfg.get('sync_send_machine_reports')
    sync = {
        'id': cfg.get('deployment_id', ''),
        'token': token,
        'upstream-url': upstream_url,
        'interval': interval,
        'send-machine-reports': send_machine_reports,
    }
    if flavors:
        flavors = [f.strip() for f in flavors.split(",")]
        sync['flavors'] = flavors
    return sync


def map_admin_auth(cfg):
    admin_auth = {}
    basic_users = cfg.get('auth_basic_users', None)
    if basic_users:
        try:
            auths = [{'username': u[0], 'password_hash': u[1]} for u in
                     map(lambda x: x.strip().split(':'), basic_users.split(','))]
        except IndexError:
            raise InvalidConfigKey("invalid config for 'auth_basic_users'")
        admin_auth['basic-auth'] = {
            'users': auths,
        }
    else:
        log('Basic authentication not configured.')

    sso_location = cfg.get('auth_sso_location', None)
    sso_public_key = cfg.get('auth_sso_public_key', None)
    lp_teams = cfg.get('auth_lp_teams', None)
    if sso_location and sso_public_key and lp_teams:
        teams = [t.strip() for t in lp_teams.split(',')]
        admin_auth['sso-auth'] = {
            'location': sso_location,
            'public-key': sso_public_key,
            'teams': teams,
        }
    else:
        log('SSO authentication not configured.')

    return admin_auth


def map_storage_config(cfg):
    patch_storage = {}
    storage_type = cfg.get('patchstore')
    if storage_type == 'file':
        pth = cfg.get('filestore_path', '').strip()
        if pth == '':
            raise MissingConfigKey("missing fs storage config")
        if not os.path.isdir(pth):
            if not os.path.exists(pth):
                os.mkdir(pth)
            else:
                raise Exception("file-backed storage path {} is not a directory".format(pth))
        return {
            'type': 'filesystem',
            'params': {
                'path': pth,
            },
        }
    elif storage_type == 'swift':
        configured = all((
            cfg.get('swift_container_name', '').strip(),
            cfg.get('swift_username', '').strip(),
            cfg.get('swift_auth_url', '').strip()))
        if not configured:
            raise MissingConfigKey("missing swift storage config")
        return {
            'type': 'swift',
            'params': {
                'username': cfg.get('swift_username', '').strip(),
                'api-key': cfg.get('swift_apikey', '').strip(),
                'auth-url': cfg.get('swift_auth_url', '').strip(),
                'domain': cfg.get('swift_domain_name', '').strip(),
                'tenant': cfg.get('swift_tenant_name', '').strip(),
                'region': cfg.get('swift_region_name', '').strip(),
                'container': cfg.get('swift_container_name', '').strip(),
            },
        }
    elif storage_type == 's3':
        configured = all((
            cfg.get('s3_bucket', '').strip(),
            cfg.get('s3_access_key_id', '').strip(),
            cfg.get('s3_secret_key', '').strip()))
        if not configured:
            raise MissingConfigKey("missing s3 storage config")
        return {
            'type': 's3',
            'params': {
                'bucket': cfg.get('s3_bucket', '').strip(),
                'endpoint': cfg.get('s3_endpoint', '').strip(),
                'region': cfg.get('s3_region', '').strip(),
                'secure': cfg.get('s3_secure', True),
                'key-id': cfg.get('s3_access_key_id', '').strip(),
                'secret-key': cfg.get('s3_secret_key', '').strip(),
            },
        }
    elif storage_type == 'postgres':
        if cfg.get('psql_url', '').strip() == '':
            raise MissingConfigKey("missing postgres storage config")
        return {
            'type': 'postgres',
            'params': {
                'db-url': cfg.get('psql_url', '').strip(),
            },
        }
    raise InvalidConfigKey(
        "invalid value for config 'patchstore': {}".format(storage_type))
