"""Utilities for retrieving livepatch-onprem resource token."""

import json
import yaml

import requests


DEFAULT_CONTRACTS_URL = 'https://contracts.canonical.com'
RESOURCE_NAME = 'livepatch-onprem'
TOKEN_FILE = '/var/lib/ubuntu-advantage/private/machine-token.json'
UA_CLIENT_CONFIG = '/etc/ubuntu-advantage/uaclient.conf'


def get_machine_token_info():
    """
    Retrieve machine-id and contract-id from the token file created
    by the ua client (if the file exists).
    """
    try:
        with open(TOKEN_FILE, 'r') as file:
            data = json.load(file)
        machine_id = data['machineTokenInfo']['machineId']
        machine_token = data['machineToken']
        return (machine_id, machine_token)
    except OSError:
        print("Could not open token file.")
        return (None, None)
    except json.JSONDecodeError:
        print("Failed to unmarshal token file.")
        return (None, None)
    except KeyError:
        print("Failed to get machine-id or token from token file.")
        return (None, None)


def get_resource_token(machine_id, machine_token, contracts_url=DEFAULT_CONTRACTS_URL):
    """
    Retrieve a resource token for the livepatch-onprem
    resource.
    """
    headers = {'Authorization': 'Bearer {}'.format(machine_token)}
    try:
        req = requests.get(
            '{}/v1/resources/{}/context/machines/{}'.format(
                contracts_url, RESOURCE_NAME, machine_id),
            headers=headers)
        data = req.json()
        return data
    except requests.exceptions.RequestException:
        print("Failed to retrieve resource token")
        return None
    except KeyError:
        print("Resource token not in contracts response.")
        return None


def get_contracts_url():
    """Parse the ua client config file and return the contracts url."""
    try:
        with open(UA_CLIENT_CONFIG, 'r') as file:
            cfg = yaml.safe_load(file)
            return cfg['contract_url']
    except KeyError:
        print("Failed to get contract url.")
        return None
    except OSError:
        print("Failed to open ua client config file.")
        return None
