import os
from shutil import copy2
import subprocess

from charmhelpers.contrib.charmsupport import nrpe
from charmhelpers.core import (
    host,
    hookenv,
    unitdata,
)
from charms import reactive

from charms.layer.livepatchserver import config

CONFIG_PATH = "/etc/livepatchd.yaml"
SNAP_NAME = "canonical-livepatch-server"
SERVICE_FILE_NAME = './resources/livepatch-server.service'
SERVICE_FILE_DEST = '/etc/systemd/system/livepatch-server.service'
BIN_FILE_NAME = './resources/livepatchd'
BIN_FILE_DEST = '/usr/local/bin/livepatchd'
SERVICE = "livepatch-server.service"

CONFIG_FLAG = 'livepatch.have-config'
DB_FLAG = 'livepatch.have-db'

KV = unitdata.kv()

@reactive.hook('leader-elected', 'leader-settings-changed')
def leader_settings():
    was_leader = KV.get('leader', False)
    is_leader = hookenv.is_leader()

    # Only apply changes if the leadership status has changed.
    if was_leader != is_leader:
        hookenv.log("Application leadership data has changed. Updating config.")
        if was_leader:
            hookenv.log("Leadership status lost.")
        else:
            hookenv.log("No longer leader.")
        check_config()


@reactive.hook('install', 'upgrade-charm')
def install_hook():
    hookenv.log("Removing newpatch snap.")
    subprocess.call(['snap', 'remove', SNAP_NAME])
    copy2(SERVICE_FILE_NAME, SERVICE_FILE_DEST)
    hookenv.status_set("maintenance", "Installing livepatch-server.")
    if livepatchd_running():
        stop_livepatchd()
    charm_dir = os.environ["JUJU_CHARM_DIR"]
    copy2(BIN_FILE_NAME, BIN_FILE_DEST)
    check_config()


@reactive.when_not("database.connected")
def database_not_connected():
    message = "✘ Needs relation to PostgreSQL service."
    hookenv.status_set("blocked", message.encode("utf-8"))
    reactive.clear_flag(DB_FLAG)
    reactive.clear_flag(CONFIG_FLAG)


@reactive.when("database.connected")
@reactive.when_not(DB_FLAG)
def set_database():
    pgsql = reactive.endpoint_from_flag('database.connected')
    hookenv.status_set('maintenance', "Configuring database.")
    cfg = hookenv.config()
    if cfg['psql_dbname']:
        pgsql.set_database(cfg['psql_dbname'])
    if cfg['psql_roles']:
        roles = [r.strip() for r in cfg['psql_roles'].split(',')]
        pgsql.set_roles(roles)


@reactive.when('database.master.available')
@reactive.when_not(DB_FLAG)
def database_available():
    pgsql = reactive.endpoint_from_flag('database.master.available')
    configure_database(pgsql)


def configure_database(pgsql):
    if pgsql and pgsql.master:
        KV.set('database.master.dsn', pgsql.master.uri)
        reactive.set_flag(DB_FLAG)
        hookenv.status_set("maintenance", "Configuring livepatch service")
        check_config()
    else:
        reactive.clear_flag(DB_FLAG)
        KV.unset('database.master.dsn')


@reactive.when('database.master.changed', DB_FLAG)
def database_changed(pgsql):
    pgsql = reactive.endpoint_from_flag('database.master.changed')
    configure_database(pgsql)
    reactive.clear_flag('database.master.changed')


@reactive.when_any('config.changed.psql_dbname', 'config.changed.psql_roles')
@reactive.when('database.connected')
def db_config_changed():
    reactive.clear_flag(DB_FLAG)
    cfg = hookenv.config()
    pgsql = reactive.endpoint_from_flag('database.connected')
    if cfg.changed('psql_dbname') and cfg['psql_dbname']:
        pgsql.set_database(cfg['psql_dbname'])
    if cfg.changed('psql_roles') and cfg['psql_roles']:
        pgsql.set_roles(cfg['psql_roles'])


@reactive.when('config.changed', DB_FLAG)
def check_config():
    """Attempt to configure livepatch service."""
    cfg = hookenv.config().copy()

    reactive.clear_flag(CONFIG_FLAG)
    pgurl = KV.get('database.master.dsn', None)
    if not pgurl:
        database_not_connected()
        return

    # KV value for sync token could be set by an action
    # that retrieves the resource token from the contracts service.
    resource_token = KV.get('sync_token', None)
    if resource_token and not cfg['sync_token']:
        cfg['sync_token'] = resource_token
        cfg['sync_upstream'] = KV.get('sync_upstream', cfg['sync_upstream'])

    cfg['psql_url'] = pgurl
    cfg['leader'] = hookenv.is_leader()

    cfg['deployment_id'] = hookenv.model_uuid()

    if cfg.get('event_bus_brokers', None) and isinstance(cfg['event_bus_brokers'], str):
        cfg['event_bus_brokers'] = cfg['event_bus_brokers'].split(',')
    else:
        cfg['event_bus_brokers'] = []

    try:
        config.check_and_write_config(cfg, CONFIG_PATH)
    except config.MissingConfigKey as exc:
        hookenv.status_set("blocked", "✘ {}".format(exc).encode('utf-8'))
        return
    except config.InvalidConfigKey as exc:
        hookenv.status_set("blocked", "✘ {}".format(exc).encode('utf-8'))
        return
    hookenv.status_set("maintenance", "Livepatch server configured.")
    reactive.set_flag(CONFIG_FLAG)
    reactive.clear_flag('config.changed')
    if not schema_updated() and not init_schema():
        hookenv.log("Schema upgrade required.")
        return
    restart_service()


@reactive.hook('update-status')
def update_status():
    service_running = host.service_running(SERVICE)
    configured = reactive.is_flag_set(CONFIG_FLAG)
    if not configured:
        # The charm is not in a configured state, the
        # status message will be set by other handlers.
        return
    if not schema_updated():
        message = "✗ Database schema needs to be upgraded. Run schema-upgrade action."
        hookenv.status_set("blocked", message.encode('utf-8'))
        return
    if service_running:
        message = "✓ Livepatch server running."
        hookenv.status_set("active", message.encode('utf-8'))
    else:
        message = "✗ Livepatch server not running."
        hookenv.status_set("maintenance", message.encode('utf-8'))


@reactive.when("website.available")
def configure_website(website):
    cfg = hookenv.config()
    http_port = cfg['port']
    website.configure(port=http_port)


@reactive.when("nrpe-external-master.available")
def setup_nagios(nagios):
    # Ask charmhelpers.contrib.charmsupport's nrpe to work out our hostname
    cfg = hookenv.config()
    http_port = cfg['port']
    hostname = nrpe.get_nagios_hostname()
    nrpe_setup = nrpe.NRPE(hostname=hostname, primary=True)
    cmd = ['/usr/lib/nagios/plugins/check_http',
           '-I', '127.0.0.1', '-p', str(http_port), "-u", "/",
           '-e', " 200 OK"]

    nrpe_setup.add_check(
        'check_http',
        'Verify livepatch service is responding',
        ' '.join(cmd)
    )
    nrpe_setup.write()


def schema_updated():
    pgurl = KV.get('database.master.dsn', None)
    if not pgurl:
        return False
    try:
        result = subprocess.check_output([
            './resources/schema-upgrade',
            '--db', pgurl,
            'check',
            './resources/schema-upgrades',
        ])
    except subprocess.CalledProcessError as e:
        return False
    return True


def init_schema():
    """Attempt schema upgrade if the db is empty."""
    pgurl = KV.get('database.master.dsn', None)
    if not pgurl:
        return False
    try:
        result = subprocess.check_output([
            './resources/schema-upgrade',
            '--db', pgurl,
            'upgrade', '--init',
            './resources/schema-upgrades',
        ])
    except subprocess.CalledProcessError as e:
        return False
    return True


def restart_service():
    hookenv.status_set("maintenance", "Re-starting service.")
    host.service_restart(SERVICE)
    update_status()


def stop_livepatchd():
    hookenv.status_set("maintenance", "Stopped service.")
    host.service_stop(SERVICE)
    update_status()


def livepatchd_running():
    return host.service_running(SERVICE)
